package de.proton.MessProgrammvonHarald;

public class Datagramm
{
    private byte[] input;

    public boolean Auto;
    public boolean AC;
    public boolean TrueRms;
    public boolean RS232;

    public boolean Diode;
    public boolean Hold;
    public boolean Delta;
    public boolean Beep;

    public boolean Overload;

    public Enum Prefix;
    public Enum Unit;
    public double PrefixFaktor;

    public Double Value;
    public Double RealValue;

    // Constructor
    public Datagramm(byte[] input)
    {
        this.input = input;
        DecodeInput();
    }

    // public Datagram(UnitsEnum _unit, PrefixEnum _prefix, Double _value)
    // {
    // Unit = _unit;
    // Prefix = _prefix;
    // Value = _value;
    // }

    public enum PrefixEnum
    {
        Nano, Micro, Milli, None, Kilo, Mega
    }

    public enum UnitsEnum
    {
        Ampere, Farad, Prozent, None, Ohm, Hertz, Volt
    }

    protected void DecodeInput()
    {
        DecodeBits();
        DecodeUnit();
        DecodeOverload();
        DecodePrefix();
        DecodeValue();
    }

    protected void DecodeValue()
    {
        if (Overload)
        {
            System.out.println("Overload");
            Value = 0.0;
        } else
        {
            Value = (1000 * DecodeNumber((byte) (input[1] << 4 | (input[2] & 0x0f))) + 100 * DecodeNumber((byte) (input[3] << 4 | (input[4] & 0x0f))) + 10 * DecodeNumber((byte) (input[5] << 4 | (input[6] & 0x0f))) + DecodeNumber((byte) (input[7] << 4 | (input[8] & 0x0f)))) * GetFactor();
            RealValue = getRealValue();
        }
    }

    protected byte DecodeNumber(byte number)
    {
        number &= 0x7f;

        switch (number)
            {
            case 0x7d:
                return 0;
            case 0x05:
                return 1;
            case 0x5b:
                return 2;
            case 0x1f:
                return 3;
            case 0x27:
                return 4;
            case 0x3e:
                return 5;
            case 0x7e:
                return 6;
            case 0x15:
                return 7;
            case 0x7f:
                return 8;
            case 0x3f:
                return 9;
            default:
                System.out.println("Unknown number " + number);
                try
                {
                    Thread.sleep(300);
                } catch (InterruptedException e)
                {                  
                    e.printStackTrace();
                }
                return 1;
            }
    }

    protected Double GetFactor()
    {
        Double factor = null;
        if ((input[1] & 0x08) == 0x08) factor = -1.0;
        if ((input[3] & 0x08) == 0x08) factor = 0.001;
        if ((input[5] & 0x08) == 0x08) factor = 0.01;
        if ((input[7] & 0x08) == 0x08) factor = 0.1;

        return factor;
    }

    protected void DecodeBits()
    {
        RS232 = GetBit(0, 0); // ???
        Auto = GetBit(0, 1);
        TrueRms = GetBit(0, 2); // ???
        AC = GetBit(0, 3);

        Diode = GetBit(9, 0);
        Beep = GetBit(10, 0);
        Hold = GetBit(11, 0);
        Delta = GetBit(11, 1);
    }

    protected boolean GetBit(int _byte, int _bit)
    {
        byte bitmask = (byte) (1 << _bit);
        return ((input[_byte] & bitmask) == bitmask);
    }

    protected void DecodeUnit()
    {
        if (GetBit(11, 3)) Unit = UnitsEnum.Farad;
        else
            if (GetBit(11, 2)) Unit = UnitsEnum.Ohm;
            else
                if (GetBit(12, 3)) Unit = UnitsEnum.Ampere;
                else
                    if (GetBit(12, 1)) Unit = UnitsEnum.Hertz;
                    else
                        if (GetBit(12, 2)) Unit = UnitsEnum.Volt;
                        else
                            if (GetBit(10, 2)) Unit = UnitsEnum.Prozent;
                            else
                            {
                                Unit = UnitsEnum.None;
                                System.out.println("Unkown Unit");
                            }
    }

    protected void DecodePrefix()
    {
        if (GetBit(9, 2))
        {
            Prefix = PrefixEnum.Nano;
            PrefixFaktor = -9;
        } else
            if (GetBit(9, 3))
            {
                Prefix = PrefixEnum.Micro;
                PrefixFaktor = -6;
            } else
                if (GetBit(10, 3))
                {
                    Prefix = PrefixEnum.Milli;
                    PrefixFaktor = -3;
                }

                else
                    if (GetBit(9, 1))
                    {
                        Prefix = PrefixEnum.Kilo;
                        PrefixFaktor = 3;
                    } else
                        if (GetBit(10, 1))
                        {
                            Prefix = PrefixEnum.Mega;
                            PrefixFaktor = 6;
                        } else
                        {
                            Prefix = PrefixEnum.None;
                            PrefixFaktor = 0;
                        }
    }

    protected void DecodeOverload()
    {
        Overload = ((input[3] & 0x07) == 0x07) && ((input[4] & 0x0f) == 0x0d) && ((input[5] & 0x07) == 0x06) && ((input[6] & 0x0f) == 0x08);
    }

    public String PrefixName()
    {
        if (Prefix == PrefixEnum.Nano) return "Nano";
        if (Prefix == PrefixEnum.Micro) return "Micro";
        if (Prefix == PrefixEnum.Milli) return "Milli";
        if (Prefix == PrefixEnum.Kilo) return "Kilo";
        if (Prefix == PrefixEnum.Mega) return "Mega";
        if (Prefix == PrefixEnum.None) return "";
        return "";
    }

    public Double getPrefixFactor()
    {
        return (Double) Math.pow(10, PrefixFaktor);
    }

    public Double getRealValue()
    {
        return Value * getPrefixFactor();
    }
}